% Codes for the paper "Quantifying the Risk of Deflation" by Kilian and Manganelli
% by Simone Manganelli
% European Central Bank
% Frankfurt
% Created: 30 October 2002
% Last modified: 18 November 2005

clear, clc

%******************************************
% Choose the model 
%    indOIL = 0 --> Inflation
%    indOIL = 1 --> Inflation & Oil Prices
%    indOIL = 2 --> Inflation & Money
indOIL = 2;
%******************************************

% Load data
US = load('j:\PapersPublished\DeflationRisk\Data\cpi_US_SA.txt');
GE = load('j:\PapersPublished\DeflationRisk\Data\cpiDE_OECD_SA_2002.txt');
JP = load('j:\PapersPublished\DeflationRisk\Data\cpiJP_OECD_SA.txt');
OIL = load('j:\PapersPublished\DeflationRisk\Data\oilpriceWTI.txt'); % Source FRED Saint Louis
M3US=load('j:\PapersPublished\DeflationRisk\Data\m1_US_SA.txt');
M2=load('j:\PapersPublished\DeflationRisk\Data\M3_DEBUBA_M2_JP_SA_OECD.txt'); TIMEM=M2(:,1); MGE=M2(:,2); MJP=M2(:,3);
FX = load('j:\PapersPublished\DeflationRisk\Data\fx_dm_yen.txt'); TIMEFX=FX(:,1); DM=FX(:,2); YEN=FX(:,3);

% Throw away October 2002 observation for US and GE and pre 1960 observations for US
% Our sample size is 1960.1 to 2002.9 monthly data
indexOIL = find(OIL(:,1)>1960); OIL = OIL(indexOIL(1:end),:);
timeOIL = OIL(:,1); cpiOIL = OIL(:,2);
OILGE = cpiOIL./DM; OILYEN = cpiOIL./YEN;

indexMUS = find(M3US(:,1)>1960); MUS=M3US(indexMUS(1:end-1),2);
indexUS = find(US(:,1)>1960);
US = US(indexUS(1:end-1),2); GE = GE(1:end-1,2); JP = JP(1:end,2);
OILUS = cpiOIL(1:end-1,1); OILGE = OILGE(1:end-1,1); OILJP = OILYEN(1:end-1,1);
DM = DM(1:end-1,1); YEN = YEN(1:end-1,1);

T = length(US);

%Compute annualised inflation rates for horizons of 12, 24, 36 months
index1Y = [T:-12:1]'; index1Y = sort(index1Y); index2Y = [T:-24:1]'; index2Y = sort(index2Y);

% US variables
US1Y = US(index1Y); US2Y = US(index2Y); 
USpi1Y = 100*log(US1Y(2:end)./US1Y(1:end-1)); USpi2Y = 100*log(US2Y(2:end)./US2Y(1:end-1))/2;

OILus1Y = OILUS(index1Y); OILus2Y = OILUS(index2Y); 
OILuspi1Y = 100*log(OILus1Y(2:end)./OILus1Y(1:end-1)); OILuspi2Y = 100*log(OILus2Y(2:end)./OILus2Y(1:end-1))/2; 

Mus1Y = MUS(index1Y); Mus2Y = MUS(index2Y);
Muspi1Y = 100*log(Mus1Y(2:end)./Mus1Y(1:end-1)); Muspi2Y = 100*log(Mus2Y(2:end)./Mus2Y(1:end-1))/2;

%FXus1Y = DM(index1Y); FXus2Y = DM(index2Y);
FXus1Y = 1./DM(index1Y); FXus2Y = 1./DM(index2Y);
FXuspiDM1Y = 100*log(FXus1Y(2:end)./FXus1Y(1:end-1)); FXuspiDM2Y = 100*log(FXus2Y(2:end)./FXus2Y(1:end-1));
%FXus1Y = YEN(index1Y); FXus2Y = YEN(index2Y);
FXus1Y = 1./YEN(index1Y); FXus2Y = 1./YEN(index2Y);
FXuspiYEN1Y = 100*log(FXus1Y(2:end)./FXus1Y(1:end-1)); FXuspiYEN2Y = 100*log(FXus2Y(2:end)./FXus2Y(1:end-1));

% German variables
GE1Y = GE(index1Y); GE2Y = GE(index2Y); 
GEpi1Y = 100*log(GE1Y(2:end)./GE1Y(1:end-1)); GEpi2Y = 100*log(GE2Y(2:end)./GE2Y(1:end-1))/2;

OILge1Y = OILGE(index1Y); OILge2Y = OILGE(index2Y); 
OILgepi1Y = 100*log(OILge1Y(2:end)./OILge1Y(1:end-1)); OILgepi2Y = 100*log(OILge2Y(2:end)./OILge2Y(1:end-1))/2; 

Mge1Y = MGE(index1Y); Mge2Y = MGE(index2Y);
Mgepi1Y = 100*log(Mge1Y(2:end)./Mge1Y(1:end-1)); Mgepi2Y = 100*log(Mge2Y(2:end)./Mge2Y(1:end-1))/2;

FXge1Y = DM(index1Y); FXge2Y = DM(index2Y);
FXgepi1Y = 100*log(FXge1Y(2:end)./FXge1Y(1:end-1)); FXgepi2Y = 100*log(FXge2Y(2:end)./FXge2Y(1:end-1));

% Japanese variables
JP1Y = JP(index1Y); JP2Y = JP(index2Y);
JPpi1Y = 100*log(JP1Y(2:end)./JP1Y(1:end-1)); JPpi2Y = 100*log(JP2Y(2:end)./JP2Y(1:end-1))/2;

OILjp1Y = OILJP(index1Y); OILjp2Y = OILJP(index2Y); 
OILjppi1Y = 100*log(OILjp1Y(2:end)./OILjp1Y(1:end-1)); OILjppi2Y = 100*log(OILjp2Y(2:end)./OILjp2Y(1:end-1))/2; 

Mjp1Y = MJP(index1Y); Mjp2Y = MJP(index2Y);
Mjppi1Y = 100*log(Mjp1Y(2:end)./Mjp1Y(1:end-1)); Mjppi2Y = 100*log(Mjp2Y(2:end)./Mjp2Y(1:end-1))/2;

FXjp1Y = YEN(index1Y); FXjp2Y = YEN(index2Y);
FXjppi1Y = 100*log(FXjp1Y(2:end)./FXjp1Y(1:end-1)); FXjppi2Y = 100*log(FXjp2Y(2:end)./FXjp2Y(1:end-1));


pi1Y = [USpi1Y, GEpi1Y, JPpi1Y]; pi2Y = [USpi2Y, GEpi2Y, JPpi2Y];
OILpi1Y = [OILuspi1Y, OILgepi1Y, OILjppi1Y]; OILpi2Y = [OILuspi2Y, OILgepi2Y, OILjppi2Y];
Mpi1Y = [Muspi1Y, Mgepi1Y, Mjppi1Y]; Mpi2Y = [Muspi2Y, Mgepi2Y, Mjppi2Y];
FXpi1Y = [FXuspiDM1Y, FXuspiYEN1Y, FXgepi1Y, FXjppi1Y]; FXpi2Y = [FXuspiDM2Y, FXuspiYEN2Y, FXgepi2Y, FXjppi2Y];

TABLE1=[]; TABLE234 = []; TABLEgarch = [];
pmax = 3; % Maximum number of lags in the SIC procedure

for y=1:2 % y=1: one year ahead forecase; y=2: two years ahead forecast
    for c=1:3 % For three countries
        X = [];
        eval(['Y=pi',int2str(y),'Y(:,c);'])
        eval(['OIL = OILpi', int2str(y),'Y(:,c);'])
        eval(['M = Mpi', int2str(y),'Y(:,c);'])
        if c==2, indMGE=find(M>-Inf); Y=Y(indMGE);OIL=OIL(indMGE);M=M(indMGE); FX=FX(indMGE); end
        
        % Use information criteria to select models
        if indOIL == 1, [SIC, LAGS]=SICselection([Y, OIL],pmax);
        elseif indOIL == 2, [SIC, LAGS]=SICselection([Y,M],pmax); 
        else [SIC, LAGS]=SICselection(Y,pmax); end
        
        LagOrder(c,y) = LAGS; pminIC(c,y) = SIC;
        
        for L=1:LAGS
            eval(['X', int2str(L), '=Y(LAGS+1-', int2str(L), ':end-', int2str(L),');'])
            eval(['X = [X, X', int2str(L), '];'])
        end
        
        if indOIL == 1
            for L=1:LAGS
                eval(['OIL', int2str(L), '=OIL(LAGS+1-', int2str(L), ':end-', int2str(L),');'])
                eval(['X = [X, OIL', int2str(L), '];'])
            end
        end
        if indOIL == 2
            for L=1:LAGS
                eval(['M', int2str(L), '=M(LAGS+1-', int2str(L), ':end-', int2str(L),');'])
                eval(['X = [X, M', int2str(L), '];'])
            end
        end
        
        Y = Y(LAGS+1:end,1); 
        X = [ones(size(Y)),X];
        
        % STEP 1 - Model the mean
        BETA = inv(X'*X) * (X'*Y); Yhat = X*BETA; Res  = Y-Yhat;
        
        % Diagnostic tests
        [LB, LBpvalue] = LjungBox(Res);%LBpvalue
        [LB2, LBpvalue2] = LjungBox(Res.^2);%LBpvalue2
        
        % STEP 2 - Compute the risk measures
        if LBpvalue2<.05 % Test for GARCH in the squared residuals
            garchOut = garchModel(Res, Yhat); 
            TABLEgarch = [TABLEgarch; c, y, garchOut.LBstd, garchOut.LBpvaluestd, garchOut.LBstd2, garchOut.LBpvaluestd2];
        end
        
        TABLE1 = [TABLE1,[LAGS; SIC; LB; LBpvalue; LB2; LBpvalue2]];
    end
end
TABLE1 = reshape(TABLE1, 18, 2);        
indOIL, TABLE1
